﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Windows.Xna;
using System.Windows.Media.Imaging;


namespace Cube3d
{
    public struct VertexPositionColor
    {
        public Vector3 Position;
        public Color Color;
        

        public VertexPositionColor(Vector3 position, Color color)
        {
            Position = position;
            Color = color;
        }

        public static readonly VertexDeclaration VertexDeclaration = new VertexDeclaration(
            new VertexElement(0, VertexElementFormat.Vector3, VertexElementUsage.Position, 0),
            new VertexElement(12, VertexElementFormat.Color, VertexElementUsage.Color, 0)
            );
    }

    public partial class MainPage : UserControl
    {
        // init the 3D scene
        private bool showPrimitives;
        // resources
        VertexBuffer vertexBuffer;
        VertexShader vertexShader;
        PixelShader pixelShader;
        Texture2D texture;

        static readonly GraphicsDevice resourceDevice = System.Windows.Graphics.GraphicsDeviceManager.Current.GraphicsDevice;

      //  Scene scene = new Scene();
        private bool cubeXAxis;
        private bool cubeYAxis;
        private bool cubeZAxis;
        private double translateZ;
        private double rotateX;
        Matrix view; // The view or camera transform
        Matrix projection; // The projection transform to convert 3D space to 2D screen space

        public MainPage()
        {
            InitializeComponent();
        }

        private void sliderX_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            rotateX = e.NewValue;
            ToolTipService.SetToolTip(sliderX, e.NewValue.ToString());

        }
        private void sliderZ_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            translateZ = e.NewValue;
            ToolTipService.SetToolTip(sliderZ, e.NewValue.ToString());

        }
        private void RotateX_Click(object sender, RoutedEventArgs e)
        {
            cubeXAxis = !cubeXAxis;
            if (cubeXAxis)
                btnRotateX.Content = "Pause X";
            else
                btnRotateX.Content = "Rotate X";

        }
        private void RotateY_Click(object sender, RoutedEventArgs e)
        {
            cubeYAxis = !cubeYAxis;
            if (cubeYAxis)
                btnRotateY.Content = "Pause Y";
            else
                btnRotateY.Content = "Rotate Y";

        }
        private void RotateZ_Click(object sender, RoutedEventArgs e)
        {
            cubeZAxis = !cubeZAxis;
            if (cubeZAxis)
                btnRotateZ.Content = "Pause Z";
            else
                btnRotateZ.Content = "Rotate Z";
        }
        private void Light_Click(object sender, RoutedEventArgs e)
        {
            if (BulbGradient.Visibility == System.Windows.Visibility.Collapsed)
            {
                BulbGradient.Visibility = System.Windows.Visibility.Visible;
                bgImage.Visibility = System.Windows.Visibility.Collapsed;
                btnLight.Content = "Background";
            }
            else
            {
                BulbGradient.Visibility = System.Windows.Visibility.Collapsed;
                bgImage.Visibility = System.Windows.Visibility.Visible;
                btnLight.Content = "Light";
            }
        }
        private void Reset_Click(object sender, RoutedEventArgs e)
        {
            cubeXAxis = false;
            cubeYAxis = false;
            cubeZAxis = false;
            btnRotateX.Content = "Rotate X";
            btnRotateY.Content = "Rotate Y";
            btnRotateZ.Content = "Rotate Z";
            sliderX.Value = 0;
            sliderZ.Value = 0;

        }
        private void Primitives_Click(object sender, RoutedEventArgs e)
        {
            showPrimitives = !showPrimitives;
            OnLoad(this, e);
        }
        private void OnSizeChanged(object sender, SizeChangedEventArgs e)
        {
            DrawingSurface surface = sender as DrawingSurface;
            float sceneAspectRatio = (float)surface.ActualWidth / (float)surface.ActualHeight;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, sceneAspectRatio, 1.0f, 100.0f);
        }
        void OnDraw(object sender, DrawEventArgs args)
        {
            Matrix position = Matrix.Identity; 
            Matrix scale = Matrix.CreateScale(1.0f); 
            float xf = 0.0f;
            float yf = 0.0f;
            float zf = 0.0f;

            if ((!cubeXAxis) && (!cubeYAxis) && (!cubeZAxis)) // first time angle
            {
                if (rotateX != 0)
                    xf = (float)rotateX;
                else
                    xf = -0.65f;
                yf = 0.00f;
                zf = 0.0f;
            }
            if (cubeXAxis) xf = MathHelper.PiOver4 * (float)args.TotalTime.TotalSeconds;
            if (cubeYAxis) yf = MathHelper.PiOver4 * (float)args.TotalTime.TotalSeconds;
            if (cubeZAxis) zf = MathHelper.PiOver4 * (float)args.TotalTime.TotalSeconds;

            Matrix rotation = Matrix.CreateFromYawPitchRoll(xf, yf, zf);

            Matrix world;
            if (translateZ != 0)
                world = rotation * Matrix.CreateTranslation(0, 0, (float)translateZ);
            else
                world = scale * rotation * position;
            
            // calculate the final coordinates to pass to the shader
            Matrix worldViewProjection = world * view * projection;
            resourceDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer, new Microsoft.Xna.Framework.Color(0, 0, 0, 0), 10.0f, 0);
            // setup vertex pipeline
            resourceDevice.SetVertexBuffer(vertexBuffer);
            resourceDevice.SetVertexShader(vertexShader);
            resourceDevice.SetVertexShaderConstantFloat4(0, ref worldViewProjection);

            resourceDevice.SetPixelShader(pixelShader);
            resourceDevice.Textures[0] = texture;
            resourceDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 12);
            args.InvalidateSurface();
        }
     
        private void OnLoad(object sender, RoutedEventArgs e)
        {
            vertexBuffer = CreateCube();
            Stream shaderStream = Application.GetResourceStream(new Uri(@"Cube3d;component/shader/shader.vs", UriKind.Relative)).Stream;
            vertexShader = VertexShader.FromStream(resourceDevice, shaderStream);
            
            Stream pixelStream = Application.GetResourceStream(new Uri(@"Cube3d;component/shader/shader.ps", UriKind.Relative)).Stream;
            pixelShader = PixelShader.FromStream(resourceDevice, pixelStream);
            
            Stream imageStream = Application.GetResourceStream(new Uri(@"Cube3d;component/scene.jpg", UriKind.Relative)).Stream;
            var image = new BitmapImage();
            image.SetSource(imageStream);
            texture = new Texture2D(resourceDevice, image.PixelWidth, image.PixelHeight, false, SurfaceFormat.Color);
            image.CopyTo(texture);
         
            Vector3 cameraPosition = new Vector3(0, 0, 5.0f); // the camera's position
            Vector3 cameraTarget = Vector3.Zero; // the place the camera is looking (towards world origin)
            view = Matrix.CreateLookAt(cameraPosition, cameraTarget, Vector3.Up);
        }
        VertexBuffer CreateCube()
        {
            // cube vertices
            var vertexCollection = new VertexPositionColor[36];

            // Front
            // A-----------------B
            // |                 |
            // |                 |
            // |                 |
            // C-----------------D
            //  
            // Back
            //    E-----------------F
            //    |                 |
            //    |                 |
            //    |                 |
            //    G-----------------H

            // front coordinates
            Vector3 cubeA = new Vector3(-0.75f, 0.75f, 0.75f);
            Vector3 cubeB = new Vector3(0.75f, 0.75f, 0.75f);
            Vector3 cubeC = new Vector3(-0.75f, -0.75f, 0.75f);
            Vector3 cubeD = new Vector3(0.75f, -0.75f, 0.75f);
            // back coordinates
            Vector3 cubeE = new Vector3(-0.75f, 0.75f, -0.75f);
            Vector3 cubeF = new Vector3(0.75f, 0.75f, -0.75f);
            Vector3 cubeG = new Vector3(-0.75f, -0.75f, -0.75f);
            Vector3 cubeH = new Vector3(0.75f, -0.75f, -0.75f);

            //  colors
            Color cRed = Color.FromNonPremultiplied(255, 0, 0, 156);
            Color cGreen = Color.FromNonPremultiplied(0, 255, 0, 156);
            Color cBlue = Color.FromNonPremultiplied(0, 0, 255, 156);
            Color cYellow = Color.FromNonPremultiplied(255, 255, 0, 156);
            Color cBlack = Color.FromNonPremultiplied(0, 0, 0, 156);
            Color cWhite = Color.FromNonPremultiplied(255, 255, 255, 156);

            if (showPrimitives)
            {
                // front 
                vertexCollection[0] = new VertexPositionColor(cubeA, cGreen);
                vertexCollection[1] = new VertexPositionColor(cubeB, cGreen);
                vertexCollection[2] = new VertexPositionColor(cubeC, cGreen);
                vertexCollection[3] = new VertexPositionColor(cubeB, cBlue);
                vertexCollection[4] = new VertexPositionColor(cubeD, cBlue);
                vertexCollection[5] = new VertexPositionColor(cubeC, cBlue);

                // back  
                vertexCollection[6] = new VertexPositionColor(cubeG, cBlue);
                vertexCollection[7] = new VertexPositionColor(cubeF, cBlue);
                vertexCollection[8] = new VertexPositionColor(cubeE, cBlue);
                vertexCollection[9] = new VertexPositionColor(cubeH, cGreen);
                vertexCollection[10] = new VertexPositionColor(cubeF, cGreen);
                vertexCollection[11] = new VertexPositionColor(cubeG, cGreen);

                // top 
                vertexCollection[12] = new VertexPositionColor(cubeE, cRed);
                vertexCollection[13] = new VertexPositionColor(cubeF, cRed);
                vertexCollection[14] = new VertexPositionColor(cubeA, cRed);
                vertexCollection[15] = new VertexPositionColor(cubeF, cYellow);
                vertexCollection[16] = new VertexPositionColor(cubeB, cYellow);
                vertexCollection[17] = new VertexPositionColor(cubeA, cYellow);

                // bottom  
                vertexCollection[18] = new VertexPositionColor(cubeH, cRed);
                vertexCollection[19] = new VertexPositionColor(cubeG, cRed);
                vertexCollection[20] = new VertexPositionColor(cubeC, cRed);
                vertexCollection[21] = new VertexPositionColor(cubeD, cYellow);
                vertexCollection[22] = new VertexPositionColor(cubeH, cYellow);
                vertexCollection[23] = new VertexPositionColor(cubeC, cYellow);

                // left 
                vertexCollection[24] = new VertexPositionColor(cubeC, cBlack);
                vertexCollection[25] = new VertexPositionColor(cubeG, cBlack);
                vertexCollection[26] = new VertexPositionColor(cubeA, cBlack);
                vertexCollection[27] = new VertexPositionColor(cubeA, cWhite);
                vertexCollection[28] = new VertexPositionColor(cubeG, cWhite);
                vertexCollection[29] = new VertexPositionColor(cubeE, cWhite);

                // right  
                vertexCollection[30] = new VertexPositionColor(cubeH, cWhite);
                vertexCollection[31] = new VertexPositionColor(cubeD, cWhite);
                vertexCollection[32] = new VertexPositionColor(cubeB, cWhite);
                vertexCollection[33] = new VertexPositionColor(cubeH, cBlack);
                vertexCollection[34] = new VertexPositionColor(cubeB, cBlack);
                vertexCollection[35] = new VertexPositionColor(cubeF, cBlack);
            }
            else
            {
                // front 
                vertexCollection[0] = new VertexPositionColor(cubeA, cGreen);
                vertexCollection[1] = new VertexPositionColor(cubeB, cGreen);
                vertexCollection[2] = new VertexPositionColor(cubeC, cGreen);
                vertexCollection[3] = new VertexPositionColor(cubeB, cGreen);
                vertexCollection[4] = new VertexPositionColor(cubeD, cGreen);
                vertexCollection[5] = new VertexPositionColor(cubeC, cGreen);

                // back  
                vertexCollection[6] = new VertexPositionColor(cubeG, cBlue);
                vertexCollection[7] = new VertexPositionColor(cubeF, cBlue);
                vertexCollection[8] = new VertexPositionColor(cubeE, cBlue);
                vertexCollection[9] = new VertexPositionColor(cubeH, cBlue);
                vertexCollection[10] = new VertexPositionColor(cubeF, cBlue);
                vertexCollection[11] = new VertexPositionColor(cubeG, cBlue);

                // top 
                vertexCollection[12] = new VertexPositionColor(cubeE, cRed);
                vertexCollection[13] = new VertexPositionColor(cubeF, cRed);
                vertexCollection[14] = new VertexPositionColor(cubeA, cRed);
                vertexCollection[15] = new VertexPositionColor(cubeF, cRed);
                vertexCollection[16] = new VertexPositionColor(cubeB, cRed);
                vertexCollection[17] = new VertexPositionColor(cubeA, cRed);

                // bottom  
                vertexCollection[18] = new VertexPositionColor(cubeH, cYellow);
                vertexCollection[19] = new VertexPositionColor(cubeG, cYellow);
                vertexCollection[20] = new VertexPositionColor(cubeC, cYellow);
                vertexCollection[21] = new VertexPositionColor(cubeD, cYellow);
                vertexCollection[22] = new VertexPositionColor(cubeH, cYellow);
                vertexCollection[23] = new VertexPositionColor(cubeC, cYellow);

                // left 
                vertexCollection[24] = new VertexPositionColor(cubeC, cBlack);
                vertexCollection[25] = new VertexPositionColor(cubeG, cBlack);
                vertexCollection[26] = new VertexPositionColor(cubeA, cBlack);
                vertexCollection[27] = new VertexPositionColor(cubeA, cBlack);
                vertexCollection[28] = new VertexPositionColor(cubeG, cBlack);
                vertexCollection[29] = new VertexPositionColor(cubeE, cBlack);

                // right  
                vertexCollection[30] = new VertexPositionColor(cubeH, cWhite);
                vertexCollection[31] = new VertexPositionColor(cubeD, cWhite);
                vertexCollection[32] = new VertexPositionColor(cubeB, cWhite);
                vertexCollection[33] = new VertexPositionColor(cubeH, cWhite);
                vertexCollection[34] = new VertexPositionColor(cubeB, cWhite);
                vertexCollection[35] = new VertexPositionColor(cubeF, cWhite);
            }
            var vb = new VertexBuffer(resourceDevice, VertexPositionColor.VertexDeclaration,
            vertexCollection.Length, BufferUsage.WriteOnly);
            vb.SetData(0, vertexCollection, 0, vertexCollection.Length, 0);
            return vb;
        } 
    }
}
